/*
 * pwm.h
 *
 *  Library created to manage pwm stuff
 *
 *  Created on: 20/11/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#ifndef __PWM_H 
#define __PWM_H

#define PCPWM1				(1 << 6)				/* ENABLE PWM1 PERIPHERAL */
#define PCLK_PWM1			(3 << 12)				/* PWM CLK PREESCALER: /8 */
#define PCLK				( SystemCoreClock / 8 )	/* REF CLOCK */
#define PWM_FREQ			( PCLK / 1000 ) 		/* REF CLOCK / Freq Hz */
#define MCR_SET				(1 << 1)
#define PWM_PIN				(1 << 8)				/* 2.4 (46) PIN */
#define PWM_PIN_MODE		(2 << 8)				/* Neither PULLUP nor PULLDOWN */

#define PWM_VOLUME_CONST	( 0.08 )				/* PWM Volume factor */

#define TCR_CNT_EN			0x00000001
#define TCR_RESET			0x00000002
#define TCR_PWM_EN			0x00000008

#define PWMSEL2				(1 << 2)
#define PWMSEL3				(1 << 3)
#define PWMSEL4				(1 << 4)
#define PWMSEL5				(1 << 5)
#define PWMSEL6				(1 << 6)
#define PWMENA1				(1 << 9)
#define PWMENA2				(1 << 10)
#define PWMENA3				(1 << 11)
#define PWMENA4				(1 << 12)
#define PWMENA5				(1 << 13)
#define PWMENA6				(1 << 14)

#define LER0_EN				(1 << 0)
#define LER1_EN				(1 << 1)
#define LER2_EN				(1 << 2)
#define LER3_EN				(1 << 3)
#define LER4_EN				(1 << 4)
#define LER5_EN				(1 << 5)
#define LER6_EN				(1 << 6)


/******************************************************************************
** Function name:		PWM_Init
**
** Descriptions:		PWM initialization, setup GPIOs to PWM1 channel 5,
**						reset counter, all latches are enabled, interrupt
**						on PWMMR0, install PWM interrupt to the VIC table.
**
** parameters:			PWM Freq, PWM volume (duty cycle based)
** Returned value:		TRUE if initialization was OK, FALSE otherwise
**
******************************************************************************/
uint32_t PWM_Init(uint32_t freq, float volume);

/******************************************************************************
** Function name:		PWM_Set
**
** Descriptions:		Sets PWM frequency and volume
**
** parameters:			PWM Freq, PWM volume (duty cycle based)
** Returned value:		None
**
******************************************************************************/
void PWM_Set(uint32_t freq, float volume);

/******************************************************************************
** Function name:		PWM_Start
**
** Descriptions:		Turns PWM signal ON, enable Channel 5 on PWM1
**
** parameters:			None
** Returned value:		None
**
******************************************************************************/
void PWM_Start(void);

/******************************************************************************
** Function name:		PWM_Start
**
** Descriptions:		Turns PWM signal OFF, disable Channel 5 on PWM1
**
** parameters:			None
** Returned value:		None
**
******************************************************************************/
void PWM_Stop(void);

#endif /* end __PWM_H */
/****************************************************************************
**                            End Of File
****************************************************************************/
