/*
 * sync.h
 *
 *  Library created to manage concurrent access
 *
 *  Created on: 18/10/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#ifndef SYNC_H_
#define SYNC_H_

/* Function name: 	initHandlers
 *
 * Description: 	Call this function to create semaphore sync for UART and Wifly.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void initHandlers();


/* Function name: 	uartLock
 *
 * Description: 	Call this function to lock UART access.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void uartLock();

/* Function name: 	uartUnLock
 *
 * Description: 	Call this function to lock UART access.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void uartUnlock();

/* Function name: 	lockUART0
 *
 * Description: 	Call this function to enable UART port 0 receive event.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void lockUART0();

/* Function name: 	wiflyLock
 *
 * Description: 	Call this function to lock Wifly usage.
 * Note:			This function must be called ONLY from Wifly driver.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void wiflyLock();


/* Function name: 	wiflyUnLock
 *
 * Description: 	Call this function to unlock Wifly usage.
 * Note:			This function must be called ONLY from Wifly driver.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void wiflyUnlock();


/* Function name: 	unlockUART0_ISR
 *
 * Description: 	Call this function to wait for UART port 0 event.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void unlockUART0_ISR();


/* Function name: 	disableUART0Int
 *
 * Description: 	Call this function to disable event from UART port 0 interrupt.
 * Notes:			This function is designed for call it from UART port 0 interrupt.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void disableUART0Int();


/* Function name: 	enableUART0Int
 *
 * Description: 	Call this function to disable sync from UART port 0 interrupt.
 *
 *
 * Parameters: 		None.
 * Return:			None.
 */
void enableUART0Int();


#endif /* SYNC_H_ */
