/*
 * tools.h
 *  Useful functions
 *
 *  Created on: 30/10/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#ifndef TOOLS_H_
#define TOOLS_H_

#define WF_BLANK_CHAR				('$')

/*****************************************************************************
** Function name:		vDelimChars
**
** Descriptions:		Extracts text portion between delimited chars.
** 						Overrides initial text by N char offset.
**
** parameters:			cStrIn, cStrOut, cDelimIni, cDelimEnd, ulOffset
** Returned value:		None
**
*****************************************************************************/
void vDelimChars(char* cStrIn, char* cStrOut, char* cDelimIni, char* cDelimEnd, uint32_t ulOffset);

/*****************************************************************************
** Function name:		cpChangeBlanks
**
** Descriptions:		Changes blank spaces of input string by '$'
**
** parameters:			Input string
** Returned value:		None
**
*****************************************************************************/
void cpChangeBlanks(char* pcText);

/*****************************************************************************
** Function name:		parseOptionData
**
** Descriptions:		Parse string in option data. Input string is formatted
** 						as !* and several options parsed by # char.
**
** parameters:			Input string, output freq, output speed, output volume
** Returned value:		None
**
*****************************************************************************/
void parseOptionData(char* string, uint32_t* freq, uint32_t* speed, float* volume);

/*****************************************************************************
** Function name:		strCopy
**
** Descriptions:		Copy source string in dest string
**
** parameters:			source string, dest string
** Returned value:		None
**
*****************************************************************************/
void strCopy(const char* source, char* dest);

/*****************************************************************************
** Function name:		strContains
**
** Descriptions:		Returns TRUE if string contains subString
**
** parameters:			string, subString
** Returned value:		TRUE if string contains subString, FALSE otherwise
**
*****************************************************************************/
uint32_t strContains(const char* string, const char* subString);

/*****************************************************************************
** Function name:		strContainsCRLF
**
** Descriptions:		Returns TRUE if input string ends with CRLF
**
** parameters:			Input string to check
** Returned value:		TRUE if CRLF ended, FALSE otherwise
**
*****************************************************************************/
uint32_t strContainsCRLF(char* string);

/*****************************************************************************
** Function name:		strRemoveCRLF
**
** Descriptions:		Remove CRLF from input string
**
** parameters:			Input string to remove CRLF
** Returned value:		None
**
*****************************************************************************/
void strRemoveCRLF(char* string);

#endif /* TOOLS_H_ */
