/*
 * uartstr.h
 *
 *  Library created to manage strings and chars through UART
 *
 *  Created on: 24/10/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#ifndef UARTSTR_H_
#define UARTSTR_H_

#define UART_DEBUG_PORT		( 3U )
#define txtCRLF				( "\r\n" )	/* CRLF */
#define CRLF_LENGTH			( 4U )		/* String length of CRLF */
#define MAX_UL_LENGTH 		( 30U )		/* Unsigned Long max text length */


/*****************************************************************************
** Function name:		printfConvert
**
** Descriptions:		Stores in *input the formatted string.
** 						This function does not send any data through UART.
** 						Only for formatting purposes.
**
** parameters:			input string, fmt, list of parameters
** Returned value:		None
**
*****************************************************************************/
void printfConvert(char *input, char *fmt, ...);

/*****************************************************************************
** Function name:		printfDebug
**
** Descriptions:		Prints through DEBUG_PORT
**
** parameters:			fmt, list of parameters
** Returned value:		None
**
*****************************************************************************/
void printfDebug(char *fmt, ...);

/*****************************************************************************
** Function name:		printfUart
**
** Descriptions:		Prints through specified port
**
** parameters:			portNum, fmt, list of parameters
** Returned value:		None
**
*****************************************************************************/
void printfUart(uint32_t portNum, char *fmt, ...);

/*****************************************************************************
** Function name:		printfInitDebug
**
** Descriptions:		Configure print through debug port
**
** parameters:			None
** Returned value:		TRUE if debug port was initialized OK
**
*****************************************************************************/
uint32_t printfInitDebug();

/*****************************************************************************
** Function name:		printfInit
**
** Descriptions:		Configure print through specified port
**
** parameters:			portNum, baudrate
** Returned value:		TRUE if port was initialized OK
**
*****************************************************************************/
uint32_t printfInit(uint32_t portNum, uint32_t baudrate);

/*****************************************************************************
** Function name:		UART0Scanf
**
** Descriptions:		Listens and retrieve chars received through UART port 0
**
** parameters:			Output string, time to listen port 0
** Returned value:		Number of characters successfully received
**
*****************************************************************************/
uint32_t UART0Scanf(char* string, uint32_t timeout);

/*****************************************************************************
** Function name:		UART3Scanf
**
** Descriptions:		Listens and retrieve chars received through UART port 3
**
** parameters:			Output string, time to listen port 3
** Returned value:		Number of characters successfully received
**
*****************************************************************************/
uint32_t UART3Scanf(char* string, uint32_t timeout);

/*****************************************************************************
** Function name:		printfFlush
**
** Descriptions:		Clean UART buffer
**
** parameters:			UART port of buffer to be cleaned
** Returned value:		None
**
*****************************************************************************/
void printfFlush(uint32_t portNum);


#endif /* UARTSTR_H_ */
