/*
 * pwm.c
 *
 *  Library created to manage pwm stuff
 *
 *  Created on: 20/11/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#include "lpc17xx.h"
#include "type.h"
#include "pwm.h"



uint32_t PWM_Init(uint32_t freq, float volume)
{
	/* Power on PWM1 with CCLK/8 CLOCK */
	LPC_SC->PCONP |= PCPWM1;
	LPC_SC->PCLKSEL0 |= PCLK_PWM1;

	LPC_PWM1->TCR = TCR_RESET;
	LPC_PWM1->PR = 0x00;
	LPC_PWM1->MCR = MCR_SET;	/* Reset TC on MR0 */


	/* Set PIN for output PWM1 on p2.4 (PIN 46) without PULLUP and PULLDOWN */
	LPC_PINCON->PINSEL4 |= PWM_PIN;
	LPC_PINCON->PINMODE4 |= PWM_PIN_MODE;

	/* Set PWM frequency and duty for signal strength */
	PWM_Set(freq, volume);

	return (TRUE);
}

void PWM_Set(uint32_t freq, float volume)
{
	uint32_t pwm_freq;

	pwm_freq = PCLK / freq;
	LPC_PWM1->MR0 = pwm_freq;
	LPC_PWM1->MR5 = (uint32_t)(volume * pwm_freq);
	LPC_PWM1->LER = LER0_EN | LER5_EN;

	return;
}

void PWM_Start(void)
{
	/* Control Mode, enable Channel 5 on PWM1 */
	LPC_PWM1->PCR = PWMENA5;

	/* Timer Counter Enable & PWM Enable */
	LPC_PWM1->TCR = TCR_CNT_EN | TCR_PWM_EN;

	return;
}

void PWM_Stop(void)
{
	LPC_PWM1->PCR = 0;
	LPC_PWM1->TCR = 0x00;		/* Stop all PWMs */

	return;
}
