/*
 * sync.c
 *
 *  Library created to manage concurrent access
 *
 *  Created on: 18/10/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#include "FreeRTOS.h"
#include "semphr.h"
#include "type.h"
#include "sync.h"


static xSemaphoreHandle handlerUART = NULL;
static xSemaphoreHandle checkWifly = NULL;
static xSemaphoreHandle wiflyUsage = NULL;
static xSemaphoreHandle uartUsage = NULL;
static uint32_t isTaskWaiting = FALSE;
static uint32_t uartIntDisabled = FALSE;


void initHandlers()
{

	if (handlerUART == NULL) {
		 vSemaphoreCreateBinary(handlerUART);
		xSemaphoreTake(handlerUART, 0);
	}

	if (uartUsage == NULL)
		uartUsage = xSemaphoreCreateMutex();

	if (checkWifly == NULL)
		checkWifly = xSemaphoreCreateMutex();

	if (wiflyUsage == NULL)
		wiflyUsage = xSemaphoreCreateMutex();
}


void uartLock()
{
	if (uartUsage != NULL)
		xSemaphoreTake(uartUsage, portMAX_DELAY);
}


void uartUnlock()
{
	if (uartUsage != NULL)
		xSemaphoreGive(uartUsage);
}


void disableUART0Int()
{
	uartIntDisabled = TRUE;
}


void enableUART0Int()
{
	uartIntDisabled = FALSE;
}


void lockUART0()
{
	if (handlerUART != NULL) {
		isTaskWaiting = TRUE;
		xSemaphoreTake(handlerUART, portMAX_DELAY);
	}
}


void wiflyLock()
{
	if (wiflyUsage != NULL)
		xSemaphoreTake(wiflyUsage, portMAX_DELAY);
}


void wiflyUnlock()
{
	if (wiflyUsage != NULL)
		xSemaphoreGive(wiflyUsage);
}


void unlockUART0_ISR() {

	portBASE_TYPE xHiguerPriorityTaskWoken = pdFALSE;

	if (uartIntDisabled != TRUE) {
		if (handlerUART != NULL && isTaskWaiting) {
			isTaskWaiting = FALSE;
			xSemaphoreGiveFromISR(handlerUART, &xHiguerPriorityTaskWoken);
		}
	}
}
