/*
 * uartcomm.c
 *
 *  Library created to manage strings and chars through UART
 *
 *  Created on: 24/10/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#include "FreeRTOS.h"
#include "LPC17xx.h"
#include "semphr.h"
#include "type.h"
#include "uart.h"
#include "delay.h"
#include "uartstr.h"
#include "config.h"
#include "sync.h"
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

extern volatile uint8_t UART0TxEmpty, UART1TxEmpty, UART3TxEmpty;
extern volatile uint8_t UART0Buffer[BUFSIZE], UART1Buffer[BUFSIZE], UART3Buffer[BUFSIZE];
extern volatile uint32_t UART0Count, UART1Count, UART3Count;

void printfSend(uint32_t portNum, char *fmt, va_list ap) {
	char strOut[BUFSIZE];

	uartLock();
	vsprintf(strOut, fmt, ap);

	LPC_UART3->IER = IER_THRE | IER_RLS;
	UARTSend(portNum, (uint8_t *)strOut, strlen(strOut));
	UART0Count = 0;
	LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;
	uartUnlock();
}

void printfDebug(char *fmt, ...) {
#ifdef DEBUG_SHOW
	va_list ap;

	va_start(ap, fmt);
	printfSend(DEBUG_UART_PORT, fmt, ap);
	va_end(ap);
#endif /* DEBUG_SHOW */
}

void printfUart(uint32_t portNum, char *fmt, ...) {
	va_list ap;

	va_start(ap, fmt);
	printfSend(portNum, fmt, ap);
	va_end(ap);
}

void printfConvert(char *input, char *fmt, ...) {
	va_list ap;

	va_start(ap, fmt);
	vsprintf(input, fmt, ap);
	va_end(ap);
}

uint32_t printfInitDebug() {
#ifdef DEBUG_SHOW
	uint32_t response = FALSE;

	response = UARTInit(DEBUG_UART_PORT, DEBUG_UART_BR);
	if (response == TRUE)
		printfFlush(DEBUG_UART_PORT);

	return response;
#endif /* DEBUG_SHOW */
}

uint32_t printfInit(uint32_t portNum, uint32_t baudrate) {


	return UARTInit(portNum, baudrate);
}


uint32_t UART0Scanf(char* string, uint32_t timeout) {
	uint32_t newCount = 0;

	uartLock();
	passiveDelay(timeout);
	vUARTDisableIRQ(0);

	if (UART0Count > 0) {
		newCount = UART0Count;
		strncpy(string, (char *)UART0Buffer, newCount);
		//string[newCount] = 0;
		printfFlush(0);

		UART0Count = 0;
	}
	else
		newCount = strlen(string);

	string[newCount] = '\0';
	vUARTEnableIRQ(0);
	uartUnlock();

	return newCount;
}

uint32_t UART3Scanf(char* string, uint32_t timeout) {
	uint32_t newCount = 0;

	/* This function retrieve user response from terminal */
	uartLock();
	passiveDelay(timeout);
	vUARTDisableIRQ(3);

	if (UART3Count > 0) {
		newCount = UART3Count;
		strncat(string , (char *)UART3Buffer, newCount);
		printfFlush(3);

		UART3Count = 0;
	}
	else
		newCount = strlen(string);

	vUARTEnableIRQ(3);
	uartUnlock();

	return newCount;
}

void printfFlush(uint32_t portNum) {

	switch (portNum){
	case 0:
		UART0Count = 0;
		UART0TxEmpty = 1;
		memset((uint8_t *)UART0Buffer, 0x00, BUFSIZE);
		break;
	case 1:
		UART1Count = 0;
		UART1TxEmpty = 1;
		memset((uint8_t *)UART1Buffer, 0x00, BUFSIZE);
		break;
	case 3:
		UART3Count = 0;
		UART3TxEmpty = 1;
		memset((uint8_t *)UART3Buffer, 0x00, BUFSIZE);
	}
}
